/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\M E M _ M N G R . C  \\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/* Routines de gestion de la memoire: Tout bloc memoire allou par les */
/* fonctions: malloc, realloc, calloc, est stock                      */
/* dans une liste chaine, avec des infos de debugage permettant de    */
/* connaitre la ligne et le nom du fichier ou il a t allou.         */
/* de plus chaque appel  free verifie si ce bloc a ete corrompu et    */
/* affiche un message de corruption.                                   */
/* quelques fonctions d'interrogation permettent  tout moment de voir */
/* si un bloc est corrompu.                                            */
/* AUTEUR: Roland SEVIN. Utilise en partie une methode de double       */
/*                       chainage publie sur:  fr.comp.sys.atari par  */
/*                       notre grand guru Jean LUSETTI merci  lui !!! */

/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
#define __EZMEM_INIT__ /* afin que: MemMngr.h ou eazy_gem.h n'activent */
                       /* pas les redefinitions de memoire, car ici il */
                       /* faut les malloc normaux non dtourns        */

/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\\\ I N C L U D E S \\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <aes.h>
#include <tos.h>
#ifdef __PUREC__
  #include <ext.h>
#else
  #include <osbind.h>
#endif

/*........ si l'on veut que la sortie des erreurs soit ...............*/
/*         directement faite dans EZ_EDIT                             */
#define __WITH_EZ_EDIT__


/* end of include.................................................. */
/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\\\ D E F I N E S  \\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/*............. structure de double lien de bloc ......................*/
typedef struct _LIST_ENTRY
       {struct _LIST_ENTRY *Flink ;
        struct _LIST_ENTRY *Blink ;
       }        LIST_ENTRY;

/*............ parametres de sortie de BlkCheck ......................*/
#define TO_FILE    1        /* parametre b sortie sur fichier erreur  */
#define TO_SCREEN  2        /* parametre b sortie sur ecran   erreur  */
#define TO_EZ_EDIT 4        /* parametre b sortie sur EZ_EDIT         */

/*............. taille des header de debut et fin de bloc .............*/
#define HEAD_SIZE  sizeof(long)*4

/*............... void  InitializeListHead(LIST_ENTRY* ListHead).......*/
#define InitializeListHead(ListHead) (\
    (ListHead)->Flink = (ListHead)->Blink = (ListHead) )
    
/*.................. void RemoveEntryList(LIST_ENTRY* Entry)...........*/
#define RemoveEntryList(Entry) {\
    LIST_ENTRY* _EX_Entry;\
    _EX_Entry = (Entry);\
    _EX_Entry->Blink->Flink = _EX_Entry->Flink;\
    _EX_Entry->Flink->Blink = _EX_Entry->Blink;\
    }
/*.........void InsertHeadList(LIST_ENTRY* ListHead, ...................*/
/*                             LIST_ENTRY* Entry)                       */
#define InsertHeadList(ListHead,Entry) \
    (Entry)->Flink = (ListHead)->Flink;\
    (Entry)->Blink = (ListHead);\
    (ListHead)->Flink->Blink = (Entry);\
    (ListHead)->Flink = (Entry)
    
/*............. structure header de memoire notant ....................*/
/*              les infos de chaque bloc reserves                      */
/*              par les fonctions malloc, calloc, realloc              */

typedef struct MEMBLK
    {size_t RealSize;         /* taille reelle y compris structure et headers   */
     size_t UserSize;         /* taille utile (celle demande)                  */
     long   MemID;            /* identificateur de memoire                      */
     LIST_ENTRY ListEntry;    /* Structure de double chainage                   */
     long   *HeadDeb;         /* pointe sur le header de debut                  */
     long   *HeadEnd;         /* pointe sur le header de debut                  */
     void   *data;            /* pointe sur les donnes utiles                  */
     long    SrceLign;        /* ligne programme ou la reservation est faite    */
     char    SrceName[128];   /* nom du fichier source ou la reservation a lieu */
    } MEM_BLK;


/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\ V A R I A B L E S    G L O B A L E S \\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*MemMngrMode 0/pas d'ecriture dans le fichier d'erreur des blocs        */
/*              desalloues.                                              */
/*            1/Il est note dans le fichier d'erreur le source et la     */
/*              ligne dans laquelle les blocs restant  desalloues et    */
/*              desalloues par cette fonction, ont ete alloues.          */
/*              cela permet de voir o ont ete alloues ces blocs memoire */
/*              non liberes.                                             */
int MemMngrMode  = 0;
/*.............. nombre de blocs memoires reservs ....................*/
long Nb_MEM_BLK  = 1;   /* 1 car celui de base toujours present        */

/*............. ofset en octets du debut structure MEM_BLK ............*/
/*                         au champ Flink.                             */
long DebStruct   = (long) &((MEM_BLK*)0)->ListEntry.Flink;

/*............. nom et chemin du fichier d'erreur .....................*/
static char StrErr[512];
static char ErrPath[256];
static char Buft[32];
/*............... reserver et initialiser un tableau de longs .........*/
/*                representant la premier bloc de depart               */
/*                            en memoire                               */

long MemBase[ (sizeof(MEM_BLK) + (HEAD_SIZE<<1) + 4) / sizeof(long)] =  /* nombre de longs  */
    {sizeof(MEM_BLK) + (HEAD_SIZE<<1) + 4, /* 0 taille reelle y compris structure et headers  */
     4,                                    /* 1 taille utile (celle demande)                 */
     0,                                    /* 2 identificateur de memoire                     */
     0,                                    /* 3 Flink pour l'instant pas de bloc suivant      */
     0,                                    /* 4 Blink pour l'instant pas de bloc precedant    */
     0,                                    /* 5 pointe sur le header de debut                 */
     0,                                    /* 6 pointe sur le header de fin                   */
     0,                                    /* 7 pointe sur les donnes utiles                 */
     0L,                                   /* 8 ligne programme ou la reservation est faite   */
     0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,      /* 9 nom du fichier ou la reservation est faite    */
     0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
     'Deb0',                               /* donnes de header de debut                    */
     'Deb1',
     'Deb2',
     'Deb3',
     'PLAF',                               /* donnes reserves utiles                      */
     'End0',                               /* donnes de header de fin                      */
     'End1',
     'End2',
     'End3'
    };
/*............... initialiser un pointeur ..............................*/
/*                sur la premiere structure                             */
/*                de bloc memoire                                       */
MEM_BLK *BaseBLK = (MEM_BLK*) &MemBase[0];

/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\\ P R O T O T Y P E S  \\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

void* MyMalloc( size_t size , long lign, char *name, long iD);
void* MyRealloc( void *ptr, size_t size , long lign, char *name, long iD);
void *MyCalloc( size_t num_elems, size_t elem_size , long lign, char *name, long iD);
void  MyFree( void *ptr , long lign, char *name);


void  InitMemMngr(int mode);
int   SetMemMngrMode(int mode);
void  MemoryCheck(int mode, long lign, char *name);
int   BlkCheck(void *test_blk, int flag, long lign, char *name);
void  ExitMemMngr(long lign, char *name);
void  WriteErreur(char *path, char *nom, char *err);
void  SetErreurName(char *path, char *nom, int mode);
void  OutErreur(char *err, long lign, char *name);
void  ToEz_edit(char *err);
int   VerifyMemBlk(MEM_BLK *pt_blk);
long  FreeMem_ID(long lign, char *name, long iD);


/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*                A   C O M M E N C E   I C I   !!!!                        */
/*              Implementation des fonctions de ce module                    */
/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/*-------------------------- SetMemMngrMode ----------------------------------*/
/* ACTION: met en place le mode par defaut de sortie des erreurs.             */
/* ENTREE: int mode: TO_FILE  l'erreur est reporte dans le fichier d'erreur  */
/*                   TO_SCREEN l'erreur est affiche  l'ecran                */
/*                   TO_EZ_EDIT l'erreur est envoye  EZ_EDIT ya plus qu'a   */
/*                              cliquer dessur pour se placer sur la ligne !! */
/*                   ces trois modes peuvent etre combins (TO_FILE|TO_SCREEN)*/
/* SORTIE: int: qui est l'ancien mode                                         */

int   SetMemMngrMode(int mode)
{int old_mode = MemMngrMode;
 MemMngrMode  = mode;
 return old_mode;
}

/*----------------------------- InitMemMngr ----------------------------------*/
/* ACTION: fonction d'initialisation generale permettant d'initialiser        */
/*         les pointeurs du premier bloc de base. Cette fonction est         */
/*         appeler en premier dans main().                                    */
/* ENTREE: int mode: TO_FILE  l'erreur est reporte dans le fichier d'erreur  */
/*                   TO_SCREEN l'erreur est affiche  l'ecran                */
/*                   TO_EZ_EDIT l'erreur est envoye  EZ_EDIT ya plus qu'a   */
/*                              cliquer dessur pour se placer sur la ligne !! */
/*                   ces trois modes peuvent etre combins (TO_FILE|TO_SCREEN)*/
/*            et modifis par la fonction: int SetMode(int mode)              */
/* SORTIE: neant                                                              */

void InitMemMngr(int mode)
{MemMngrMode  = mode;
 BaseBLK->HeadDeb = (long*)((size_t)MemBase + sizeof(MEM_BLK)); 
 BaseBLK->HeadEnd = (long*)((size_t)MemBase + sizeof(MEM_BLK) + HEAD_SIZE + sizeof(long));
 BaseBLK->data    = (void*)((size_t)MemBase + sizeof(MEM_BLK) + HEAD_SIZE); 
 InitializeListHead( &BaseBLK->ListEntry ) ;
 SetErreurName("","ERREUR.ERR", 0);
}

/*------------------------- MyCalloc -----------------------------------*/
void *MyCalloc( size_t num_elems, size_t elem_size , long lign, char *name, long iD)
{void *adr = MyMalloc( (size_t) (num_elems*elem_size) , lign, name, iD);
 if (adr) memset(adr, 0, num_elems*elem_size);
 return adr;
}

/*------------------------- MyMalloc -----------------------------------*/
void *MyMalloc( size_t size , long lign, char *name, long iD)
{if (size>0) 
    {size_t   len_blk  = (size_t)sizeof(MEM_BLK) + (size_t)(HEAD_SIZE<<1) + (size_t)((size+1)&-2L);
     MEM_BLK *new_blk  = malloc(len_blk);
     long    *head     = 0;
     if (new_blk==0) 
        {sprintf(StrErr,"Error %s %ld: allocation impossible taille demande %ld taille manager %ld",name,lign,size,len_blk);
         WriteErreur(ErrPath, 0, StrErr);
         return 0;
        }
     new_blk->RealSize = len_blk;
     new_blk->UserSize = size;
     new_blk->MemID    = iD;
     new_blk->ListEntry.Flink = 0;
     new_blk->ListEntry.Blink = 0;
     new_blk->HeadDeb  = (long*)((size_t)new_blk + (size_t)sizeof(MEM_BLK));
     new_blk->HeadEnd  = (long*)((size_t)new_blk + (size_t)sizeof(MEM_BLK) + (size_t)HEAD_SIZE + (size_t)((size+1L)&-2L) );
     new_blk->data     = (void*)((size_t)new_blk + (size_t)sizeof(MEM_BLK) + (size_t)HEAD_SIZE);
     new_blk->SrceLign = lign;
     memcpy(new_blk->SrceName, name,126); new_blk->SrceName[127]=0;
     head              = new_blk->HeadDeb;
     head[0] = 'Deb0';
     head[1] = 'Deb1';
     head[2] = 'Deb2';
     head[3] = 'Deb3';
     head              = new_blk->HeadEnd;
     head[0] = 'End0';
     head[1] = 'End1';
     head[2] = 'End2';
     head[3] = 'End3';
     
     InsertHeadList( &BaseBLK->ListEntry, &new_blk->ListEntry );
     Nb_MEM_BLK++;
     return new_blk->data;
    }
 sprintf(StrErr,"Error %s %ld: taille bloc mmoire allou demand < 0 %ld !!!",name,lign,size);
 WriteErreur(ErrPath, 0, StrErr);
 return 0;
}

/*------------------------- MyFree -------------------------------------*/
void MyFree( void *ptr , long lign, char *name)
{if (ptr!=0)
    {MEM_BLK *pt_blk       = BaseBLK;
     MEM_BLK *ToKill_blk   = (MEM_BLK*) ((size_t)ptr - HEAD_SIZE - sizeof(MEM_BLK));
     long             nb   = 0;
     
     /*........................ chercher bloc ..........................*/
     /*                      tester si corrompu                         */
     /*                   avant liberation memoire                      */
     while (nb<Nb_MEM_BLK)
        {if (pt_blk != BaseBLK &&
             pt_blk == ToKill_blk
            )
            {if (VerifyMemBlk(ToKill_blk)) WriteErreur(ErrPath, 0, StrErr);
             RemoveEntryList(&ToKill_blk->ListEntry);
             free (ToKill_blk);
             Nb_MEM_BLK--;
             return;
            }
         pt_blk     = (MEM_BLK*)((long)pt_blk->ListEntry.Flink - DebStruct);
         nb++;
        } /* end while (pt_blk->NextBlk && nb<Nb_MEM_BLK) */
      strcpy(StrErr, "Error ");
      strcat(StrErr, name);
      strcat(StrErr," ");
      strcat(StrErr, ltoa(lign, Buft,10));
      strcat(StrErr, ": free d'un bloc non alloue par notre programme !!!");
      WriteErreur(ErrPath, 0, StrErr);
     } /*endif (ptr!=0) */
  return;
}


/*------------------------- MyRealloc ------------------------------------*/
/*    realloc(NULL, size) est quivalent  malloc(size).                  */
/*    realloc(ptr, 0) est quivalent  free(ptr).                         */


void* MyRealloc( void *ptr, size_t size , long lign, char *name, long iD)
{if (ptr==0 && size == 0)                 return 0;
 if (ptr==0)                              return MyMalloc(size, lign, name, iD);
 if (size==0) {MyFree( ptr, lign, name ); return 0;}
 else
    {MEM_BLK        *pt_blk   = BaseBLK;
     MEM_BLK *ToRealloc_blk   = (MEM_BLK*) ((size_t)ptr - HEAD_SIZE - sizeof(MEM_BLK));
     size_t         len_blk   = sizeof(MEM_BLK) + (HEAD_SIZE<<1) + ((size+1)&-2L);
     MEM_BLK    *new_blk      = 0;
     long                nb   = 0;
     long             *head   = 0;
     /*........ chercher bloc ...........*/
     while (nb<Nb_MEM_BLK)
        {if (pt_blk != BaseBLK &&
             pt_blk == ToRealloc_blk
            )
            {RemoveEntryList(&ToRealloc_blk->ListEntry);
             new_blk = realloc(ToRealloc_blk, len_blk);
             if (new_blk==0)
                {return 0;
                }
             else
                {new_blk->RealSize = len_blk;
                 new_blk->UserSize = size;
                 new_blk->MemID    = iD;
                 new_blk->HeadDeb  = (long*)((size_t)new_blk + sizeof(MEM_BLK));
                 new_blk->HeadEnd  = (long*)((size_t)new_blk + sizeof(MEM_BLK) + HEAD_SIZE + ((size+1)&-2L) );
                 new_blk->data     = (void*)((size_t)new_blk + sizeof(MEM_BLK) + HEAD_SIZE);
                 new_blk->SrceLign = lign;
                 memcpy(new_blk->SrceName, name,32); new_blk->SrceName[31]=0;
                 head              = new_blk->HeadDeb;
                 head[0] = 'Deb0';
                 head[1] = 'Deb1';
                 head[2] = 'Deb2';
                 head[3] = 'Deb3';
                 head              = new_blk->HeadEnd;
                 head[0] = 'End0';
                 head[1] = 'End1';
                 head[2] = 'End2';
                 head[3] = 'End3';
                 InsertHeadList( &BaseBLK->ListEntry, &new_blk->ListEntry );
                 return new_blk->data;
                }
            }
         pt_blk     = (MEM_BLK*)((long)pt_blk->ListEntry.Flink - DebStruct);
         nb++;
        } /* end while (pt_blk->NextBlk && nb<Nb_MEM_BLK) */
     strcpy(StrErr,"Error ");
     strcat(StrErr, name);
     strcat(StrErr," ");
     strcat(StrErr, ltoa(lign, Buft,10));
     strcat(StrErr,": realloc d'un bloc non allou par notre programme !!");
     WriteErreur(ErrPath, 0, StrErr);
    } /*endif (ptr!=0) */
  return 0;
}

/*------------------------ VerifyMemBlk -------------------------------------*/
int VerifyMemBlk(MEM_BLK *pt_blk)
{long *head        = pt_blk->HeadDeb;
 int is_corrupt    = 0;
 *StrErr           = 0;
 if (head[0] != 'Deb0'||
     head[1] != 'Deb1'||
     head[2] != 'Deb2'||
     head[3] != 'Deb3'
    )
    {is_corrupt |= 1;
    }
 head         = pt_blk->HeadEnd;
 if (head[0] != 'End0'||
     head[1] != 'End1'||
     head[2] != 'End2'||
     head[3] != 'End3'
    )
    {is_corrupt |= 2;
    }
 if (is_corrupt)
    {strcpy(StrErr,"Error ");
     strcat(StrErr, pt_blk->SrceName);
     strcat(StrErr," ");
     strcat(StrErr, ltoa(pt_blk->SrceLign, Buft,10));
     strcat(StrErr,": le bloc allou  cette ligne est corrompu ");
     if (is_corrupt&1) strcat(StrErr," au dbut ");
     if (is_corrupt&2) strcat(StrErr,"  la fin ");
     return 1;
    }
 else
    {return 0;
    }
}
/*-------------------- ExitMemMngr ------------------------------------------*/
/* ACTION: libere tous les blocs alloues de la liste chaine. Cette fonction */
/*          appeler en fin de programme permet d'etre certain que toute la  */
/*         memoire sera libre en fin de programme. Normalement il ne       */
/*         devrait  pas y avoir de bloc restant  desallouer...              */
/* ENTREE: long lign: ligne du source ou est appele la fonction (mettre     */
/*                    __LINE__   )                                           */
/*         char *name: nom du source ou est appele la fonction. (mettre     */
/*                    __FILE__                                               */
/* SORTIE: Neant.                                                            */
/*                                                                           */
/* NOTE:   si des erreurs sont trouves lors de la desallocation, elles sont */
/*         reportes dans le fichier d'erreurs et ou affiches  l'ecran     */
/*         selon le mode mis en place par la fonction:                       */
/*                       void InitMemMngr(int mode)  o   int mode           */
/*          = TO_FILE  l'erreur est reporte dans le fichier d'erreur        */
/*          = TO_SCREEN l'erreur est affiche  l'ecran                      */
/*          = TO_EZ_EDIT l'erreur est envoye  EZ_EDIT ya plus qu'a cliquer!*/
/*            ces trois modes peuvent etre combins (TO_FILE|TO_SCREEN)      */
/*            et modifis par la fonction: int SetMode(int mode)             */

void ExitMemMngr(long lign, char *name)
{MEM_BLK *pt_blk       = BaseBLK;
 long             nb   = 0;
 /*........ chercher bloc ...........*/
  while (nb<Nb_MEM_BLK)
    {if (pt_blk != BaseBLK)
        {/*.............. si corruption sortir message d'erreur .......*/
         if (VerifyMemBlk(pt_blk)) 
            {if (MemMngrMode&TO_FILE)   WriteErreur(ErrPath, 0, StrErr);
             if (MemMngrMode&TO_SCREEN) printf(StrErr);
             if (MemMngrMode&TO_EZ_EDIT) ToEz_edit(StrErr);
            }
         /*............. de toutes les faons sortir ..................*/
         /*              message d'oubli de desalocation               */
         if (MemMngrMode)
            {strcpy(StrErr,"Error ");
             strcat(StrErr, pt_blk->SrceName);
             strcat(StrErr," ");
             strcat(StrErr, ltoa(pt_blk->SrceLign, Buft,10));
             strcat(StrErr,": dsallocation de ce bloc 0x");
             strcat(StrErr, ltoa((long)pt_blk->data, Buft,16));
             strcat(StrErr, " Size ");
             strcat(StrErr, ltoa((long)pt_blk->UserSize, Buft,10));
             strcat(StrErr," oubli");
             if (MemMngrMode&TO_FILE)   WriteErreur(ErrPath, 0, StrErr);
             if (MemMngrMode&TO_SCREEN) printf(StrErr);
             if (MemMngrMode&TO_EZ_EDIT) ToEz_edit(StrErr);
            }
         RemoveEntryList(&pt_blk->ListEntry);
         free (pt_blk);
         Nb_MEM_BLK--;
        }
     else
        {nb++;
        }
     pt_blk     = (MEM_BLK*)((long)pt_blk->ListEntry.Flink - DebStruct);
    } /* end while (pt_blk->NextBlk && nb<Nb_MEM_BLK) */
  if (nb>1)
     {strcpy(StrErr,"Error ExitMemMngr()");
      strcat(StrErr, name);
      strcat(StrErr," ");
      strcat(StrErr, ltoa(lign, Buft,10));
      strcat(StrErr,": Erreur interne grave plus de bloc que possible !!!");
      if (MemMngrMode&TO_FILE)   WriteErreur(ErrPath, 0, StrErr);
      if (MemMngrMode&TO_SCREEN) printf(StrErr);
      if (MemMngrMode&TO_EZ_EDIT) ToEz_edit(StrErr);
     }
  return;
}

/*-------------------- FreeMem_ID -------------------------------------------*/
/* ACTION: libere tous les blocs alloues de la liste chaine, dont l'identi- */
/*         est commun.                                                       */
/* ENTREE: long lign: ligne du source ou est appele la fonction (mettre     */
/*                    __LINE__   )                                           */
/*         char *name: nom du source ou est appele la fonction. (mettre     */
/*                    __FILE__                                               */
/*         long Id: identificateur des blocs  liberer.                      */
/* SORTIE: long: nombre de blocs liberes.                                    */
/*                                                                           */
/* NOTE:   si des erreurs sont trouves lors de la desallocation, elles sont */
/*         reportes dans le fichier d'erreurs et ou affiches  l'ecran     */
/*         selon le mode mis en place par la fonction:                       */
/*                       void InitMemMngr(int mode)  o   int mode           */
/*          = TO_FILE  l'erreur est reporte dans le fichier d'erreur        */
/*          = TO_SCREEN l'erreur est affiche  l'ecran                      */
/*          = TO_EZ_EDIT l'erreur est envoye  EZ_EDIT ya plus qu'a cliquer!*/
/*            ces trois modes peuvent etre combins (TO_FILE|TO_SCREEN)      */
/*            et modifis par la fonction: int SetMode(int mode)             */

long FreeMem_ID(long lign, char *name, long iD)
{MEM_BLK *pt_blk       = BaseBLK;
 long             nb   = 0;
 long        nb_free   = 0;
 
 lign++;    /* juste pour calmer le compil */
 name++;    /* juste pour calmer le compil */
 /*........ chercher bloc ...........*/
  while (nb<Nb_MEM_BLK)
    {if (pt_blk != BaseBLK)
        {if (pt_blk->MemID==iD)
            {/*.............. si corruption sortir message d'erreur .......*/
             if (VerifyMemBlk(pt_blk)) 
                {if (MemMngrMode&TO_FILE)    WriteErreur(ErrPath, 0, StrErr);
                 if (MemMngrMode&TO_SCREEN)  printf(StrErr);
                 if (MemMngrMode&TO_EZ_EDIT) ToEz_edit(StrErr);
                }
             RemoveEntryList(&pt_blk->ListEntry);
             free (pt_blk);
             Nb_MEM_BLK--;
             nb_free++;
            }
         else
            {nb++;
            }
        } /* endif (pt_blk->MemID==Id) */
     else
        {nb++;
        }
     pt_blk     = (MEM_BLK*)((long)pt_blk->ListEntry.Flink - DebStruct);
    } /* end while (pt_blk->NextBlk && nb<Nb_MEM_BLK) */
  
  return nb_free;
}

/*-------------------- MemoryCheck -------------------------------------------*/
/* ACTION: scanne tous les blocs allous au moment de l'appel et verifie si   */
/*         ils sont corrompus et ecrit dans le fichier d'erreur si c'est le   */
/*         cas.                                                               */
/* ENTREE: int mode: TO_FILE  l'erreur est reporte dans le fichier d'erreur  */
/*                   TO_SCREEN l'erreur est affiche  l'ecran                */
/*                   TO_EZ_EDIT l'erreur est envoye  EZ_EDIT ya plus qu'a   */
/*                              cliquer dessur pour se placer sur la ligne !! */
/*                   ces trois modes peuvent etre combins (TO_FILE|TO_SCREEN)*/
/*         long lign: ligne du source ou est appele la fonction (mettre      */
/*                    __LINE__   )                                            */
/*         char *name: nom du source ou est appele la fonction. (mettre      */
/*                    __FILE__                                                */
/* SORTIE: neant.                                                             */

void MemoryCheck(int mode, long lign, char *name)
{MEM_BLK *pt_blk       = BaseBLK;
 long             nb   = 0;
 /*........ chercher bloc ...........*/
 while (nb<Nb_MEM_BLK)
       {if (VerifyMemBlk(pt_blk))
           {if (mode&TO_FILE)   WriteErreur(ErrPath, 0, StrErr);
            if (mode&TO_SCREEN) printf(StrErr);
           }
        pt_blk     = (MEM_BLK*)((long)pt_blk->ListEntry.Flink - DebStruct);
        nb++;
       } /* end while (pt_blk->NextBlk && nb<Nb_MEM_BLK) */
 if (Nb_MEM_BLK-1)
    {strcpy(StrErr,"Found ");
     strcat(StrErr, name);
     strcat(StrErr," ");
     strcat(StrErr, ltoa(lign, Buft,10));
     strcat(StrErr,": Nombre de blocs allous lors verification = ");
     strcat(StrErr, ltoa(Nb_MEM_BLK-1, Buft,10 ));
     if (mode&TO_FILE)   WriteErreur(ErrPath, 0, StrErr);
     if (mode&TO_SCREEN) printf(StrErr);
    }
  
  return;
}


/*----------------------- BlkCheck -------------------------------------------*/
/* ACTION: verifie si l'adresse memoire donne en entree est corrompue.       */
/*         cette fonction est utile afin de savoir si apres appel  une       */
/*         fonction, celle ci abime une zone memoire sur laquelle elle        */
/*         doit travailler.                                                   */
/* ENTREE: void *test_ptr: pointeur initialise par une des fonctions          */
/*                         malloc, calloc, realloc, dont il faut verifier     */
/*                         l'integrite de la zone memoire pointe.            */
/*         int mode: TO_FILE  l'erreur est reporte dans le fichier d'erreur  */
/*                   TO_SCREEN l'erreur est affiche  l'ecran                */
/*                   TO_EZ_EDIT l'erreur est envoye  EZ_EDIT ya plus qu'a   */
/*                              cliquer dessur pour se placer sur la ligne !! */
/*                   ces trois modes peuvent etre combins (TO_FILE|TO_SCREEN)*/
/*         long lign: ligne du source ou est appele la fonction (mettre      */
/*                    __LINE__   )                                            */
/*         char *name: nom du source ou est appele la fonction. (mettre      */
/*                    __FILE__                                                */
/* RETOUR: 1/ une erreur a ete trouve.                                       */
/*         0/ pas d'erreur trouve.                                           */

int BlkCheck(void *test_ptr, int mode, long lign, char *name)
{MEM_BLK *pt_blk       = BaseBLK;
 MEM_BLK *test_blk     = (MEM_BLK*) ((size_t)test_ptr - HEAD_SIZE - sizeof(MEM_BLK));
 long             nb   = 0;
 /*........ chercher bloc ...........*/
 while (nb<Nb_MEM_BLK)
       {if (pt_blk != BaseBLK &&
            pt_blk == test_blk
           )
           {if (VerifyMemBlk(pt_blk))
               {if (mode&TO_FILE)   WriteErreur(ErrPath, 0, StrErr);
                if (mode&TO_SCREEN) printf(StrErr);
                strcpy(StrErr,"Error ");
                strcat(StrErr, name);
                strcat(StrErr," ");
                strcat(StrErr, ltoa(lign, Buft,10));
                strcat(StrErr,": Bloc corrompu dtect lors de ce Block Check  ");
                if (mode&TO_FILE)    WriteErreur(ErrPath, 0, StrErr);
                if (mode&TO_SCREEN)  printf(StrErr);
                if (mode&TO_EZ_EDIT) printf(StrErr);
                return 1;
               }
            else 
               {return 0;
               }
           }
         pt_blk     = (MEM_BLK*)((long)pt_blk->ListEntry.Flink - DebStruct);
         nb++;
        } /* end while (pt_blk->NextBlk && nb<Nb_MEM_BLK) */
  strcpy(StrErr,"Error ");
  strcat(StrErr, name);
  strcat(StrErr," ");
  strcat(StrErr, ltoa(lign, Buft,10));
  strcat(StrErr,": vrification d'un bloc non allou par notre programme !!");
  if (mode&TO_FILE)    WriteErreur(ErrPath, 0, StrErr);
  if (mode&TO_SCREEN)  printf(StrErr);
  if (mode&TO_EZ_EDIT) printf(StrErr);
  return 1;
}


/*---------------------- SetErreurName ----------------------------*/
/* ACTION: permet de donner le chemin et le nom du fichier         */
/*         d'erreurs par defaut.                                   */
/* ENTREE: char *path, char *nom: pointe sur le chemin et le nom   */
/*                                du fichier d'erreurs. Si nom = 0 */
/*                                alors path doit etre complet     */
/*                                sinon nom est concatene  path   */
/*         int mode: 1/ si il existe un fichier d'erreur portant   */
/*                      le nom de celui donn en entre alors il   */
/*                      est effac.                                */
/*                   0/ si il existe un fichier d'erreur portant   */
/*                      le nom de celui donn en entre alors il   */
/*                      n'est pas effac.                          */
/* SORTIE: neant.                                                  */

void SetErreurName(char *path, char *nom, int mode)
{ErrPath[0]=0;
 if (path) strcat(ErrPath, path);
 if (nom)  strcat(ErrPath, nom);
 if (mode)
    {int f_hand = (int)Fopen( ErrPath, FO_READ);
     if (f_hand  >= 0)
        {long len_fic = filelength(f_hand);
         Fclose (f_hand);
         if (len_fic>-1) Fdelete(ErrPath);
        }
    }
}

/*---------------------- WriteErreur ------------------------------*/
/* ACTION: ajoute la chaine: char *err dans le fichier d'erreurs   */
/*         dont le nom et le chemin sont donns en entre.         */
/* ENTREE: char *path, char *nom: pointe sur le chemin et le nom   */
/*                                du fichier d'erreurs. Si nom = 0 */
/*                                alors path doit etre complet     */
/*                                sinon nom est concatene  path   */
/* char *err: pointe sur la chaine  rajouter au fichier d'erreurs */
/* SORTIE: neant.                                                  */

void WriteErreur(char *path, char *nom, char *err)
{FILE *f;
 /*........... rajouter eventuel retour chariot ...................*/
 if (err>=StrErr && err<StrErr+sizeof(StrErr)-2)
    {strcat(err,"\r\n");
    }
 if (nom) strcat (path, nom);
 f = fopen( path, "ab");
 if (f) 
    {fwrite(err, strlen(err), 1, f);
     fclose(f);
    }
}

/*-------------------------------- OutErreur --------------------------------*/
/* ACTION: ajoute la chaine: char *err dans le fichier d'erreurs             */
/*         par defaut et ou l'affiche selon mode de sortie par defaut        */
/* ENTREE: char *err: pointe sur la chaine  rajouter au fichier             */
/*         d'erreurs par defaut et ou  afficher.                            */
/*         long lign: ligne du source ou est appele la fonction (mettre     */
/*                    __LINE__   )                                           */
/*         char *name: nom du source ou est appele la fonction. (mettre     */
/*                    __FILE__                                               */
/* SORTIE: neant.                                                            */
/* NOTE:   La chaine:  char *err est ajoute dans le fichier d'erreurs et ou */
/*         affiche  l'ecran selon le mode mis en place par la fonction:    */
/*                       void InitMemMngr(int mode)  o   int mode           */
/*          = TO_FILE  l'erreur est reporte dans le fichier d'erreur        */
/*          = TO_SCREEN l'erreur est affiche  l'ecran                      */
/*          = TO_EZ_EDIT l'erreur est envoye  EZ_EDIT ya plus qu'a cliquer!*/
/*            ces trois modes peuvent etre combins (TO_FILE|TO_SCREEN)      */
/*            et modifis par la fonction: int SetMode(int mode)             */

void OutErreur(char *err, long lign, char *name)
{ strcpy(StrErr, "Logout ");
  strcat(StrErr, name);
  strcat(StrErr, " ");
  strcat(StrErr, ltoa(lign, Buft,10));
  strcat(StrErr, " ");
  strcat(StrErr, err);
 if (MemMngrMode&TO_FILE)    WriteErreur(ErrPath, 0, err);
 if (MemMngrMode&TO_SCREEN)  printf(err);
 if (MemMngrMode&TO_EZ_EDIT) ToEz_edit(StrErr);
}

/*------------------------------- WriteMessage -----------------------------*/
/* ACTION: sort un message d'erreur ou d'autre nature:                       */
/*              => dans la fenetre Logout d'EZ_GEM: where = TO_EZ_EDIT,      */
/*              => dans le fichier d'erreurs:       where = TO_FILE          */
/*              =>  l'ecran:                       where = TO_SCREEN        */
/*               sachant que les trois modes peuvent etre combines  l'aide  */
/*               de l'operateur ou | exple: TO_SCREEN | TO_EZ_EDIT | TO_FILE */
/* ENTREE: long where: flag determinant ou doit se faire la sortie           */
/*                     (voir explication ci dessus)                          */
/*         long lign: ligne du source ou est appele la fonction (mettre     */
/*                    __LINE__   )                                           */
/*         char *name: nom du source ou est appele la fonction. (mettre     */
/*                    __FILE__                                               */
/*              si zero alors le message ne sera pas localis et precede de  */
/*              la mention exple: "Logout C:\CHEMIN\SOURCE.C  4092           */
/*              permettant  un editeur de texte sachant gerer ces infos de  */
/*              retrouver la ligne dans le code source ou le message a ete   */
/*              genere.                                                      */
/*         const char *format: chaine decrivant le format de la liste        */
/*                             d'arguments qui va suivre (meme syntaxe que   */
/*                             printf() )                                    */
/*         ... : liste d'arguments comme celle de printf()                   */
/* SORTIE: neant.                                                            */
/* NOTE:   la chaine generee ne devra pas depasser 16000 octets.             */

void WriteMessage(long where, long lign, char *name, const char *format, ...)
{char* mess   = MyMalloc(4096*4, __LINE__, __FILE__ , 0L);
 char* pt_buf = 0;
 mess[0]      = 0;
 if (mess==0)         return;
 pt_buf = mess;
 /*........... ecrire localisation du message ...................*/
 if (name)
    {strcpy(pt_buf, "Logout ");               pt_buf = pt_buf + 7;
     strcpy(pt_buf, name);                    pt_buf = pt_buf + strlen(pt_buf);
     strcpy(pt_buf, " ");                     pt_buf = pt_buf + 1;
     strcpy(pt_buf, ltoa(lign, Buft,10));     pt_buf = pt_buf + strlen(pt_buf);
     strcpy(pt_buf, " ");                     pt_buf = pt_buf + 1;
    }
 /*............ y rajouter les arguments ........................*/
 {
  va_list args;
  va_start( args,  format);
  vsprintf(pt_buf, format, args);
  va_end( args);
 }
  /*............ sortir le resultat ..............................*/
 if (where&TO_FILE)    WriteErreur(ErrPath, 0, mess);
 if (where&TO_SCREEN)  printf(mess);
 if (where&TO_EZ_EDIT) ToEz_edit(mess);
 MyFree( mess ,  __LINE__, __FILE__);
}

/*------------------------------------ ToEz_edit ----------------------------*/

void ToEz_edit(char *data)
{
 #ifdef __WITH_EZ_EDIT__
 
 #define VA_START   0x4711
 #define MESG       0
 #define APPL       1
 #define SUPL       2
 char*        Timed_MemAdd(size_t len, long time);
 extern int Appl_id;
 int ez_edit         = appl_find("EZ_EDIT ");
 if (ez_edit>0)
    {int msgbuf[8];
     /*.......... ne pas oublier lorque vous recevrez ..............*/
     /*           de la part de l'application cliente (ici EZ_EDIT) */
     /*           VA_AKCNW 0x4738, avec en pipe[3] et pipe[4]       */
     /*           une adresse =  pt de desaloue ce bloc             */
     /*char *pt = MyMalloc( strlen(data)+8 , __LINE__, __FILE__ , -1L);*/
     char *pt = Timed_MemAdd(strlen(data)+8, 20*10); /* desalocation automatique */
     if (pt)
        {strcpy(pt  , "-d ");
         strcpy(pt+3, data);
         msgbuf[MESG] = VA_START;           /*   message VA_START                 */
         msgbuf[APPL] = Appl_id;            /*   iddentificateur de l'appelant    */
         msgbuf[SUPL] = 0;                  /*   message sans surplus             */
         *(long*)(&msgbuf[3])=(long)pt;
         appl_write(ez_edit, 16, msgbuf );
        }
    }
 #endif
}

#undef __EZMEM_INIT__  /* liberer les malloc dtourns        */
