/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\ T I M S T R . C \\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/* Par Roland SEVIN :  upsis@club-internet.fr                          */
/* Module de gestion de memoire temporaire. Ce module offre des        */
/* fonctions de reservation de memoire temporaire. CAD avec            */
/* desalocation automatique au bout d'un certain temps.                */
/* L'utilite s'est fait ressentir au cours de VA_START.                */
/* Probleme: Je reserve de la memoire pour envoyer une chaine VA_START */
/*            un programme client.                                    */
/*           Celui ci devrait logiquement me retourner un VA_AKCNW     */
/*           me signalant qu'il n'a plus besoin de ma chaine afin que  */
/*           je puisse la detruire.                                    */
/*           Or certains programmes impolis ne le font jamais et l'on  */
/*           se retrouve avec une fuite de memoire, car l'on ne libere */
/*           jamais le bloc alou  la chaine.                         */
/*           Alors pour remedier a cela, je cree un zone de memoire    */
/*           temporaire laissant le temps au client de reagir, et      */
/*           la zone est detruite automatiquement quoi qu'il arrive    */
/*           au bout d'un certain temps.                               */
/* MODE D'EMPLOI: 1) inclure TIMSTR.C dans votre projet.               */
/*                2) Placez: #include "TIMSTR.H" en tete du fichier .C */
/*                   dans lequel vous aurez  vous servir des          */
/*                   fonctions de reservation de memoire temporaire.   */
/*                3) faites en sorte que la fonction:                  */
/*                   Timed_StrCheck(0) soit appelee toutes les 200 ms  */
/*                                     en permanence dans le programme */
/*                                     boucle evnt_multi ou autre ...  */
/*                4) Utilisez une des trois fonctions de reservation   */
/*                   de memoire temporaire:  Timed_StrAdd();           */
/*                                           Timed_MemAdd()            */
/*                                           Timed_PtrAdd()            */
/*                   en precisant la duree de vie dans le parametre:   */
/*                   long time en 200me de secondes (10 pour 2 s)     */
/*                5) A la fin de votre programme, appelez la fonction: */
/*                   Timed_StrCheck(1) (avec 1 comme paramtre) afin   */
/*                   de liberer les eventuelles zones de memoire       */
/*                   temporaire encore actives.                        */
/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\\\ I N C L U D E S \\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include <stdlib.h>
#include <string.h>

/*.................... inclure EAZY_GEM.H pour avoir MemMngr ..........*/
/*           sinon cela devrait fonctionner juste avec stdlib.h        */

/*#include "EAZY_GEM.H" */ /* inclure les definitions des fonctions que le */
                       /* que le module EAZY_GEM.C met  notre         */
                       /* notre disposition                            */
                       /* et inclure les definitions des variables     */
                       /* globales  EAZY_GEM                           */

/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\\\ V A R I A B L E S \\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/*....................... structure maillon de chaine .................*/
/*                        pour chainage avant et arriere               */
typedef struct TIMED_Str
{       struct TIMED_Str*     m_Prev;
        struct TIMED_Str*     m_Next;
        long           m_create_time;
        char*          m_Str;
}TIMED_STR;

/*...................... globales .....................................*/
 long m_ApplClock                 = 0;             /* application  pour l'instant reu zero EVNT_TIMER */
 long m_TMD_STR_Timer             = 0;             /* pas de timer VA_START install                    */
 long m_TMD_STR_Nb                = 0;             /* nombre de VA_START                                */
 TIMED_STR* m_pTMD_STR_Frst       = 0;             /* premiere chaine VA_START                          */

/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\\\ P R O T O T Y P E S \\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
char*        Timed_StrAdd(char *str,   long time);
char*        Timed_MemAdd(size_t size, long time);
char*        Timed_PtrAdd(char* pt,    long time);
void         Timed_StrCheck(int all);


/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\ PRIVES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
TIMED_STR*   Timed_StrNew(char *str, long time);
TIMED_STR*   Timed_StrNewBySize(size_t size, long time);
TIMED_STR*   Timed_StrNewByPtr(char* pt, long time);
void         Timed_StrDelete(TIMED_STR *TMD_STR_ToDel);
TIMED_STR*   Timed_StrDel(TIMED_STR* TMD_STR_ToDel);
extern void Pause_EZ(long t) ;


/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*                A   C O M M E N C E   I C I   !!!!                   */
/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\ PUBLIQUES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/*--------------------------- Timed_StrAdd --------------------------------*/
/* ACTION: cree une copie temporaire de la chaine: char *str place en     */
/*         entree. Cette copie se detruit automatiquement au bout de       */
/*         long time temps.                                                */
/* ENTREE: char *str: pointeur sur la chaine  copier.                     */
/*         long time: duree de vie de la copie en x 200 ms                 */
/*                   (exemple: 10 pour 2s de duree de vie)                 */
/* SORTIE: char*  pointe sur la copie temporaire cree.                     */

char*  Timed_StrAdd(char *str, long time)
{TIMED_STR* new_TMD_STR = Timed_StrNew(str, m_ApplClock + time);
 if (new_TMD_STR == 0)        return 0;
 
 /*.... inserer nouvelle en position zero .......*/
 /*     son m_Prev est donc forcement  zero     */
 /*     car y en a pas avant                     */
 /*     son m_Next est sur celle qui va etre     */
 /*     apres CAD la premiere actuelle           */
 new_TMD_STR->m_Next = m_pTMD_STR_Frst;
 /*.... Mettre  jour le Prev de celle qui ......*/
 /*     etait la premiere sur celle qui val la   */
 /*     preceder CAD la nouvelle                 */
 if (m_pTMD_STR_Frst != 0)
    {m_pTMD_STR_Frst->m_Prev = new_TMD_STR;
    }
 /*........ mettre la premiers  jour sur la nouvelle ....*/
 m_pTMD_STR_Frst = new_TMD_STR;
 m_TMD_STR_Timer = m_ApplClock+time;
 m_TMD_STR_Nb++;
 return new_TMD_STR->m_Str;
}

/*--------------------------- Timed_MemAdd --------------------------------*/
/* ACTION: cree une zone memoire temporaire de taille:  size_t size        */
/*         Cette zone memoire automatiquement au bout delong time temps.   */
/* ENTREE: size_t size: taille de la zone memoire a creer.                 */
/*         long time: duree de vie de la zone memoire en x 200 ms          */
/*                   (exemple: 10 pour 2s de duree de vie)                 */
/* SORTIE: char*  pointe sur la zone memoire temporaire cree.              */

char*  Timed_MemAdd(size_t size, long time)
{TIMED_STR* new_TMD_STR = Timed_StrNewBySize(size, m_ApplClock + time);
 if (new_TMD_STR == 0)        return 0;
 
 /*.... inserer nouvelle en position zero .......*/
 /*     son m_Prev est donc forcement  zero     */
 /*     car y en a pas avant                     */
 /*     son m_Next est sur celle qui va etre     */
 /*     apres CAD la premiere actuelle           */
 new_TMD_STR->m_Next = m_pTMD_STR_Frst;
 /*.... Mettre  jour le Prev de celle qui ......*/
 /*     etait la premiere sur celle qui val la   */
 /*     preceder CAD la nouvelle                 */
 if (m_pTMD_STR_Frst != 0)
    {m_pTMD_STR_Frst->m_Prev = new_TMD_STR;
    }
 /*........ mettre la premiers  jour sur la nouvelle ....*/
 m_pTMD_STR_Frst = new_TMD_STR;
 m_TMD_STR_Timer = m_ApplClock+time;
 m_TMD_STR_Nb++;
 return new_TMD_STR->m_Str;
}

/*--------------------------- Timed_PtrAdd --------------------------------*/
/* ACTION: Donne  un pointeur un caractre temporaire de long time temps. */
/*         Au bout de long time temps ce pointeur sera libere              */
/* ENTREE: char* pt: pointeur auquel donner le caractre temporaire        */
/*         long time: duree de vie du pointeur en x 200 ms                 */
/*                   (exemple: 10 pour 2s de duree de vie)                 */
/* SORTIE: char*  pointe sur le pointeur.                                  */

char*  Timed_PtrAdd(char* pt, long time)
{TIMED_STR* new_TMD_STR = Timed_StrNewByPtr(pt, m_ApplClock + time);
 if (new_TMD_STR == 0)        return 0;
 
 /*.... inserer nouvelle en position zero .......*/
 /*     son m_Prev est donc forcement  zero     */
 /*     car y en a pas avant                     */
 /*     son m_Next est sur celle qui va etre     */
 /*     apres CAD la premiere actuelle           */
 new_TMD_STR->m_Next = m_pTMD_STR_Frst;
 /*.... Mettre  jour le Prev de celle qui ......*/
 /*     etait la premiere sur celle qui val la   */
 /*     preceder CAD la nouvelle                 */
 if (m_pTMD_STR_Frst != 0)
    {m_pTMD_STR_Frst->m_Prev = new_TMD_STR;
    }
 /*........ mettre la premiers  jour sur la nouvelle ....*/
 m_pTMD_STR_Frst = new_TMD_STR;
 m_TMD_STR_Timer = m_ApplClock+time;
 m_TMD_STR_Nb++;
 return new_TMD_STR->m_Str;
}

/*--------------------------- Timed_StrCheck --------------------------------*/
/* ACTION: Cette fonction est le noyau des zones de memoire temporaire, et   */
/*         doit etre appelee tres regulierement par un timer toutes les      */
/*         200 ms. (placer un timer evenementiel de 200 ms et appeler cette  */
/*         fonction chaque evenement timer).                                 */
/*         cette fonction verifie si une des zones memoire temporaire a fini */
/*         son temps et la detruit si besoin.                                */
/*         Dans EZ_GEM vous n'avez pas  le faire car deja fait.             */
/* ENTREE: int all: si 0/ seules les zones memoires ayant fini leur  temps   */
/*                        seront detruites.                                  */
/*                  si 1/ toutes les zones memoires temporaires seront       */
/*                        detruites. ( faire en sortie de programme)        */
/*                                   (dans EZ_GEM deja fait)                 */
/* ATTENTION: Normalement cette fonction est deja appelee par EASY_GEM ainsi */
/*            que l'appel de fin: Timed_StrCheck(1); vous n'aurez donc qu'a  */
/*            utiliser: Timed_StrAdd() Timed_MemAdd(), Timed_PtrAdd()        */

void    Timed_StrCheck(int all /* == 0 */)
{if ( all   || 
     (m_pTMD_STR_Frst!=0 && m_ApplClock > m_TMD_STR_Timer)
    )
    {
     long nb     = 0;
     TIMED_STR* pTMD_STR_ToTest = m_pTMD_STR_Frst;
     while(pTMD_STR_ToTest)
       {if (m_TMD_STR_Timer >= pTMD_STR_ToTest->m_create_time||all)
           {if (all) Pause_EZ(5);
            pTMD_STR_ToTest = Timed_StrDel(pTMD_STR_ToTest);
            nb++;
           }
        else
           {pTMD_STR_ToTest = pTMD_STR_ToTest->m_Next;
           }
       } /* end while */
    }
 m_ApplClock++;
}

/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\ PRIVEES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*------------------------ Timed_StrNew ------------------------------*/
TIMED_STR* Timed_StrNew(char *str, long time)
{TIMED_STR* TMD_Str     = malloc(sizeof(TIMED_STR));
 if (TMD_Str==0)                                        return 0;
 TMD_Str->m_Next        = 0;
 TMD_Str->m_Prev        = 0;
 TMD_Str->m_create_time = time;
 TMD_Str->m_Str         = malloc(strlen(str)+4);
 if (TMD_Str->m_Str==0) {free(TMD_Str);                 return 0;}
 strcpy(TMD_Str->m_Str, str);
 return TMD_Str;
}

/*------------------------ Timed_StrNewBySize -------------------------*/
TIMED_STR* Timed_StrNewBySize(size_t size, long time)
{TIMED_STR* TMD_Str     = malloc(sizeof(TIMED_STR));
 if (TMD_Str==0)                                        return 0;
 TMD_Str->m_Next        = 0;
 TMD_Str->m_Prev        = 0;
 TMD_Str->m_create_time = time;
 TMD_Str->m_Str         = malloc(size);
 if (TMD_Str->m_Str==0) {free(TMD_Str);                 return 0;}
 return TMD_Str;
}

/*------------------------ Timed_StrNewByPtr -------------------------*/
TIMED_STR* Timed_StrNewByPtr(char* pt, long time)
{TIMED_STR* TMD_Str     = malloc(sizeof(TIMED_STR));
 if (TMD_Str==0)                                        return 0;
 TMD_Str->m_Next        = 0;
 TMD_Str->m_Prev        = 0;
 TMD_Str->m_create_time = time;
 TMD_Str->m_Str         = pt;
 if (TMD_Str->m_Str==0) {free(TMD_Str);                 return 0;}
 return TMD_Str;
}

/*------------------------ Timed_StrDelete -------------------------*/
void      Timed_StrDelete(TIMED_STR *TMD_Str)
{if (TMD_Str->m_Str!=0) free(TMD_Str->m_Str);
 TMD_Str->m_Str = 0;
 free(TMD_Str);
}

/*--------------------------- Timed_StrDel --------------------------------*/
TIMED_STR*   Timed_StrDel(TIMED_STR* TMD_STR_ToDel)
{if (TMD_STR_ToDel==0)      return 0;
 else
    {/*........... relever le suivant .........................*/
     TIMED_STR* pTMD_STR_Next = TMD_STR_ToDel->m_Next;
     /*....... si Une avant celle que l'on efface .............*/
     /*     mettre  jour son pointeur next sur celle d'apres  */
     if (TMD_STR_ToDel->m_Prev!=0) 
        {TMD_STR_ToDel->m_Prev->m_Next =  pTMD_STR_Next;
        }
     /*....... si Une apres celle que l'on efface .............*/
     /*     mettre  jour son pointeur prev sur celle d'avant  */
     if (TMD_STR_ToDel->m_Next!=0) 
        {TMD_STR_ToDel->m_Next->m_Prev =  TMD_STR_ToDel->m_Prev;
        }
     /*........ y en a pas avant ni apres c'est donc ..........*/
     /*         la derniere que l'on efface                    */
     if (TMD_STR_ToDel->m_Prev==0 && TMD_STR_ToDel->m_Next==0)
        {m_pTMD_STR_Frst = 0;
        }
     Timed_StrDelete(TMD_STR_ToDel);
     m_TMD_STR_Nb--;
     if (m_TMD_STR_Nb==0 )m_pTMD_STR_Frst=0;
     return pTMD_STR_Next;
    }
}


