/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\ UN_STACK.C \\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\    module de GESTION \\\\\\\\\\\\\\\\\\\\\\\\\*/
/*                          d'une pile UNDO                            */

/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\\\ I N C L U D E S \\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*................ INCLUDES DES AUTRES MODULES DU PROGRAMME ...........*/
#include "EAZY_GEM.H"  /* inclure les definitions des fonctions que le */
                       /* que le module EAZY_GEM.C met  notre         */
                       /* notre disposition                            */
                       /* et inclure les definitions des variables     */
                       /* globales  EAZY_GEM                           */
/* end of include...................................................... */

/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
/*                A   C O M M E N C E   I C I   !!!!                   */
/*              Definition des fonctions de ce module                   */
/*\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/*------------------ UNDO_CreateStack ----------------------------------*/
/* ACTION: reserve et initialise une zone de memoire permettant de      */
/*         de faire fonctionner une pile de UNDO                        */
/* ENTREE: long Nb_UNDO: nombre de UNDO possibles dans cette pile.      */
/*                       au delas de ce nombre le premier UNDO entr    */
/*                       sera perdu.                                    */
/* RETOUR: UNDO_STACK *: un pointeur sur la pile UNDO. Ce pointeur est  */
/*                       a conserver, car a dtruire en fin de          */
/*                       programme, par la fonction: UNDO_DestroyStack()*/
/*                       Ce pointeur vous sera tout le temps demand    */
/*                       pour l'utilisation des autre fonctions UNDO    */
/*                       SI zero c'est que la memoire pour creer cette  */
/*                       pile UNDO n'a pu etre reservee.                */

UNDO_STACK *UNDO_CreateStack(long Nb_UNDO)
{/*............. on prend un enregistrement de undo de plus ..........*/
/*            initialise sur des 0, qui sera celui de fin             */
 long len_mem         = sizeof(UNDO_STACK) + sizeof(UNDO_REC)*(Nb_UNDO+2);
 UNDO_STACK *U_Stack  = malloc(len_mem);
 if (U_Stack) {memset(U_Stack,0,len_mem);
               U_Stack->Pt_deb   = (UNDO_REC*) (sizeof(UNDO_STACK) + (long)U_Stack);
               U_Stack->Pt_end   = (UNDO_REC*) (len_mem            + (long)U_Stack);
               U_Stack->Rec_cur  = -1;
               U_Stack->Rec_nbr  = Nb_UNDO;
              }
 return U_Stack;
}

/*-------------------- UNDO_DestroyStack -----------------------------*/
/* ACTION: fonction de  destruction d'une pile UNDO. Cette fonction   */
/*         est  utiliser en fin de programme (ou avant si vous       */
/*         n'avez plus besoin de cette pile) pour detruire une pile   */
/*         UNDO et liberer la memoire reservee par sa creation.       */
/* ENTREE: UNDO_STACK *U_Stack: adresse de la pile UNDO a detruire.   */
/* RETOUR: toujours zero.                                             */
 
UNDO_STACK *UNDO_DestroyStack(UNDO_STACK *U_Stack) 
{long i;
if (U_Stack)
   {for (i=0; i <= U_Stack->Rec_cur; i++)
        {if (U_Stack->Pt_deb[i].Type == AUTO_ALLOC) free(U_Stack->Pt_deb[i].Data1.Dszstr);
        }
    free (U_Stack);
   }
 return 0;
}


/*--------------------- UNDO_Push -------------------------------------*/
/* ACTION: empile une action UNDO dans la pile de UNDO.                */
/* ENTREE: char *token: pointeur sur la chaine de caractere du nom de  */
/*                      l'action UNDO a empiler(8 caracteres maxi)     */
/*         long data1; premier parametre de cette action undo          */
/*         long data2; deuxieme parametre de cette action undo         */
/*         long data3; troisieme parametre de cette action undo        */
/*         UNDO_STACK *U_Stack: adresse de la pile UNDO concernee.     */
/* RETOUR: indice de l'enregistrement de cette action UNDO.            */

long UNDO_Push(char *token, INT16 type, long data1, long data2, long data3, UNDO_STACK *U_Stack)
{UNDO_REC *undo_rec;
 /*............. si le type est AUTO_ALLOC, il faut alouer ........*/
 /*              la memoire de UNDO et y copier les                */
 /*              donnes pointe par long data1 et de              */ 
 /*              longueur long data2.                              */
 if (type == AUTO_ALLOC)
    {char *mem = malloc(data2 + 2);
     if (mem)
        {memcpy(mem, (char*)data1, data2);
         mem[data2] = 0;
         data1      = (long)mem;
        }
     else 
        {return U_Stack->Rec_cur;
        }
    }
 
 
 /*........... si on est en fin de pile .............................................*/
 /*             faire la place au sommet                                             */
 /*            en detruisant le premier                                              */
 if (U_Stack->Rec_cur >= U_Stack->Rec_nbr)
    {/*....... si l'enregistrement qui doit peter est autoaloue .....................*/
     /*         il faut liberer memoire alouee                                       */
     if (U_Stack->Pt_deb[0].Type == AUTO_ALLOC) free(U_Stack->Pt_deb[0].Data1.Dszstr);
     memmove(U_Stack->Pt_deb, 
             &U_Stack->Pt_deb[1],
             sizeof(UNDO_REC)*(U_Stack->Rec_nbr+1));
     undo_rec = &U_Stack->Pt_deb[U_Stack->Rec_cur];
    }
 /*........... sinon placer l'enregistrement .......................................*/
 /*             la position suivante                                               */
 /*            et liberer la memoire eventuellement alouee par                      */
 /*            l'enregistrement dont il prend la place                              */
 else
    {U_Stack->Rec_cur++;
     undo_rec = &U_Stack->Pt_deb[U_Stack->Rec_cur];
     if (U_Stack->Pt_deb[U_Stack->Rec_cur].Type == AUTO_ALLOC) free(U_Stack->Pt_deb[U_Stack->Rec_cur].Data1.Dszstr);
    }
 /*............. placer texte token de cet enregistrement .........*/
 strncpy(undo_rec->Token, token, 8); 
 /*............. placer type de cet enregistrement ................*/
 undo_rec->Type = type;
 /*............. placer parametre 1 de cet enregistrement .........*/
 undo_rec->Data1.Dlong = data1;
 /*............. placer parametre 2 de cet enregistrement .........*/
 undo_rec->Data2.Dlong = data2;
 /*............. placer parametre 3 de cet enregistrement .........*/
 undo_rec->Data3.Dlong = data3;
 
 return U_Stack->Rec_cur;
}

/*---------------------  UNDO_Pop -------------------------------------*/
/* ACTION: depile une Action UNDO,                                     */
/* ENTREE: UNDO_STACK *U_Stack: adresse de la pile UNDO concernee.     */
/* SORTIE: UNDO_REC*: pointeur sur l'action UNDO depilee.              */
  
UNDO_REC* UNDO_Pop(UNDO_STACK *U_Stack)
{long rec_cur = U_Stack->Rec_cur;
 if (rec_cur>=0)
    {U_Stack->Rec_cur--;
     return &U_Stack->Pt_deb[rec_cur];
    }
 else
    {return 0;
    } 
}


/*---------------------  UNDO_Del -------------------------------------*/
/* long UNDO_Del(long U_FirstIndice, long U_LastIndice, UNDO_STACK *U_Stack); */
/*---------------------  UNDO_Ins -------------------------------------*/
/* long UNDO_Ins(UNDO_STACK *U_Dest, UNDO_STACK *U_Srce, long U_DestIndice, long U_NbIndice, ); */
/*---------------------  UNDO_GetLast ---------------------------------*/
/* long UNDO_GetLast(UNDO_STACK *U_Stack); */
/*---------------------  UNDO_Flush -----------------------------------*/
/* void UNDO_Flush(UNDO_STACK *U_Stack);   */
